<?php

namespace Drupal\cosign\EventSubscriber;

use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Drupal\Core\Routing\TrustedRedirectResponse;
use Symfony\Component\HttpKernel\Event\FilterResponseEvent;
use Symfony\Component\HttpKernel\KernelEvents;
use Drupal\cosign\CosignFunctions\CosignSharedFunctions;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Routing\RedirectDestinationInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Cosign Subscriber class.
 */
class CosignSubscriber implements EventSubscriberInterface {

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;
  /**
   * Destination Interface.
   *
   * @var \Drupal\Core\Routing\RedirectDestinationInterface
   */
  protected $destination;

  /**
   * Constructs a new Cosign Subscriber object.
   */
  public function __construct(ConfigFactoryInterface $configFactory, RedirectDestinationInterface $destination) {
    $this->configFactory = $configFactory;
    $this->destination = $destination;
  }

  /**
   * Creates the static objects.
   *
   * @param \Symfony\Component\DependencyInjection\ContainerInterface $container
   *   The Container interface.
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('redirect.destination')
    );
  }

  /**
   * Checks Redirection.
   *
   * @param \Symfony\Component\HttpKernel\Event\FilterResponseEvent $event
   *   Event.
   */
  public function checkRedirection(FilterResponseEvent $event) {
    $request_uri = $event->getRequest()->getRequestUri();
    $referer = $event->getRequest()->server->get('HTTP_REFERER');
    // $secure_site = $this->configFactory->get('cosign.settings')->get(
    // 'cosign_secure_site');
    $response = $event->getResponse();
    $username = CosignSharedFunctions::cosignUserMasqueradeCheck();

    if (empty($username) & ($response->getStatusCode() == 403 || strpos($request_uri, 'user/login') || strpos($request_uri, 'user/register'))) {
      if (!CosignSharedFunctions::cosignIsHttps()
        // && strpos($response->getTargetUrl(), 'ttps://')
      ) {
        // Settargeturl will not work if not an event from a redirect
        // the controller takes care of a straight user/login url
        // we can intercept the redirect route here and throw to https
        // there may be a better way to handle this
        // if (!strpos($response->getTargetUrl(), 'user/login') ||
        // !strpos($response->getTargetUrl(), 'user/register')) {.
        $https_url = 'https://' . $_SERVER['HTTP_HOST'] . $request_uri;
        $response->setTrustedTargetUrl($https_url);
        // }
      }
      else {
        $destination = $this->destination->getAsArray()['destination'];
        global $base_path;
        $base_url = rtrim($_SERVER['HTTP_HOST'], '/');
        if (!$username) {
          $request_uri = $this->configFactory->get('cosign.settings')->get('cosign_login_path') . '?cosign-' . $_SERVER['HTTP_HOST'] . '&https://' . $base_url;
          if ($destination == $base_path . 'user/login' || $destination == $base_path . 'user/register') {
            if (empty($referer)) {
              $destination = $base_path;
            }
            else {
              $destination = str_replace('https://' . $base_url, '', $referer);
            }
          }
          $request_uri = $request_uri . $destination;
        }
        else {
          CosignSharedFunctions::cosignUserStatus($username);
          if ($request_uri == $base_path . 'user/login' || $request_uri == $base_path . 'user/register') {
            $request_uri = $referer;
          }
          else {
            $request_uri = $destination;
          }
        }
        if (empty($request_uri) || strpos($request_uri, 'user/logout')) {
          $request_uri = '/';
        }
        if ($response instanceof TrustedRedirectResponse) {
          $response->setTrustedTargetUrl($request_uri);
        }
        else {
          $event->setResponse(new TrustedRedirectResponse($request_uri));
        }
      }
    }
  }

  /**
   * Retrieve Subscribed events.
   */
  public static function getSubscribedEvents() {
    $events[KernelEvents::RESPONSE][] = ['checkRedirection'];
    return $events;
  }

}
