<?php

namespace Drupal\cosign\Routing;

use Drupal\Core\Routing\RouteSubscriberBase;
use Symfony\Component\Routing\RouteCollection;
use Drupal\Core\Config\ConfigFactoryInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Listens to the dynamic route events.
 */
class RouteSubscriber extends RouteSubscriberBase {
  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Constructs a Route Subscriber Object.
   *
   * @param Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   The configFactory Interface.
   */
  public function __construct(ConfigFactoryInterface $configFactory) {
    $this->configFactory = $configFactory;
  }

  /**
   * Creates the static objects.
   *
   * @param \Symfony\Component\DependencyInjection\ContainerInterface $container
   *   The Container interface.
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function alterRoutes(RouteCollection $collection) {
    if ($this->configFactory->get('cosign.settings')->get('cosign_ban_password_resets')) {
      if ($route = $collection->get('user.pass')) {
        $route->setRequirement('_access', 'FALSE');
      }
      if ($route = $collection->get('user.reset')) {
        $route->setRequirement('_access', 'FALSE');
      }
    }
  }

}
